/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 $Author: nobunaga $
 $Revision: 1.9 $
 $Date: 2007/12/02 19:38:56 $
*/

/*
* Author : Boris Mikhailenko <stg34@stargazer.dp.ua>
*/
//---------------------------------------------------------------------------
#ifndef IA_AUTH_C_H
#define IA_AUTH_C_H

#ifndef WIN32
#include <netinet/in.h>
#else
#include <winsock2.h>
#endif

#include <string>
#include <map>

#include <boost/thread.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/cstdint.hpp>
#include <boost/date_time/posix_time/posix_time_types.hpp>

extern "C" {
#include "blowfish.h"
}

#include "ia_packets.h"
#include "ia_types.h"

//---------------------------------------------------------------------------
class IA_CLIENT_PROT
{
    public:
        ~IA_CLIENT_PROT();

        void start();
        void stop();

        void setStatusCb(const StatusCb & cb) { m_statusCb = cb; }
        void setStatCb(const StatCb & cb) { m_statCb = cb; }
        void setMessageCb(const MessageCb & cb) { m_messageCb = cb; }
        void setErrorCb(const ErrorCb & cb) { m_errorCb = cb; }
        void setDirNamesCb(const DirNamesCb & cb) { m_dirNamesCb = cb; }

        void setReconnect(bool r) { m_reconnect = r; }

        void connect();
        void disconnect();
        bool waitConnect(const boost::posix_time::time_duration & timeout);
        bool waitDisconnect(const boost::posix_time::time_duration & timeout);
        void setEnabledDirs(const bool * selectedDirs);
        void getStat(LOADSTAT * ls);
        bool isAuthorized() const;
        std::string getError() const;
        int getErrorCode() const;

    private:
        IA_CLIENT_PROT(const std::string & serverName,
                       uint16_t port,
                       const std::string & login,
                       const std::string & password);
        void m_run();
        void m_netRecv();
        void m_netSend(int n);
        void m_checkTimeout();
        int m_prepareNet(const std::string & serverName, uint16_t port);
        int m_determinatePacketType(const char * name) const;

        void m_process_CONN_SYN_ACK_8(const void * buffer);
        void m_process_ALIVE_SYN_8(const void * buffer);
        void m_process_DISCONN_SYN_ACK_8(const void * buffer);
        void m_process_FIN_8(const void * buffer);
        void m_process_INFO_8(const void * buffer);
        void m_process_ERROR(const void * buffer);

        size_t m_prepare_CONN_SYN_8(void * buffer, size_t size);
        size_t m_prepare_CONN_ACK_8(void * buffer, size_t size);
        size_t m_prepare_ALIVE_ACK_8(void * buffer, size_t size);
        size_t m_prepare_DISCONN_SYN_8(void * buffer, size_t size);
        size_t m_prepare_DISCONN_ACK_8(void * buffer, size_t size);

        void m_fillHdr8(char * buffer);
        int m_send(char * buffer, size_t len);
        int m_recv(char * buffer, size_t len);

        LOADSTAT m_loadStat;
        int m_phase;
        unsigned long m_phaseTime;
        mutable std::string m_strError;
        mutable int m_codeError;
        bool m_running;

        BLOWFISH_CTX m_ctxPass;
        BLOWFISH_CTX m_ctxHdr;

        bool m_selectedDirs[DIR_NUM];

        std::string m_login;

        struct sockaddr_in m_servAddr;

        bool m_firstConnect;
        bool m_reconnect;
        int m_sockr;
        uint32_t m_userTimeout;
        uint32_t m_aliveTimeout;
        uint32_t m_rnd;

        StatusCb m_statusCb;
        StatCb m_statCb;
        MessageCb m_messageCb;
        ErrorCb m_errorCb;
        DirNamesCb m_dirNamesCb;

        std::map<std::string, int> m_packetTypes;

        mutable boost::mutex m_mutex;
        boost::condition_variable m_cond;
        boost::thread m_thread;

        friend class IAFactory;
};

typedef boost::shared_ptr<IA_CLIENT_PROT> IAClientPtr;

#endif //IA_AUTH_C_H
