#ifndef __IA_FACTORY_H__
#define __IA_FACTORY_H__

#include <string>

#include <boost/cstdint.hpp>

#include "ia_types.h"

class IA;

class IAFactory {
    public:
        IAFactory(const std::string & serverName,
                  uint16_t port,
                  const std::string & login,
                  const std::string & password,
                  bool reconnect = false);
        ~IAFactory() {}

        void setStatusCb(const StatusCb & cb) { m_statusCb = cb; }
        void setStatCb(const StatCb & cb) { m_statCb = cb; }
        void setMessageCb(const MessageCb & cb) { m_messageCb = cb; }
        void setErrorCb(const ErrorCb & cb) { m_errorCb = cb; }
        void setDirNamesCb(const DirNamesCb & cb) { m_dirNamesCb = cb; }

        void setServerName(const std::string & serverName) { m_serverName = serverName; }
        void setPort(uint16_t port) { m_port = port; }
        void setLogin(const std::string & login) { m_login = login; }
        void setPassword(const std::string & password) { m_password = password; }
        void setReconnect(bool r) { m_reconnect = r; }

        IA * newIA() const;

    private:
        std::string m_serverName;
        uint16_t m_port;
        std::string m_login;
        std::string m_password;
        bool m_reconnect;

        StatusCb m_statusCb;
        StatCb m_statCb;
        MessageCb m_messageCb;
        ErrorCb m_errorCb;
        DirNamesCb m_dirNamesCb;
};

#endif
