/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __IA_MAIN_WINDOW__
#define __IA_MAIN_WINDOW__

#include <vector>
#include <string>

#ifndef Q_MOC_RUN
#include <boost/shared_ptr.hpp>
#endif

#include <QWidget>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QGroupBox>
#include <QTableView>
#include <QGridLayout>
#include <QIcon>
#include <QAction>
#include <QSettings>
#include <QStringList>
#include <QMap>
#include <QCloseEvent>
#include <QTimer>

#ifndef Q_MOC_RUN
#include "path_provider.h"
#include "ia_indicator.h"
#include "ia_tray_icon.h"
#include "ia.h"
#include "ia_factory.h"
#include "ia_stats_model.h"
#include "messagehistorywidget.h"
#include "messagesmodel.h"
#endif

typedef boost::shared_ptr<IA_CLIENT_PROT> ProtoPtr;

class IAMainWindow : public QWidget {

    Q_OBJECT

public:
    IAMainWindow(QSettings & s, const PathProvider & pp);
    virtual ~IAMainWindow();

public slots:
    void toggleConnect(bool state);
    void onSettings();
    void onErrorOccured(const QString & errorMessage);
    void onConnect();
    void onDisconnect();
    void onHide();
    void onShow();
    void onTrayActivated(QSystemTrayIcon::ActivationReason reason);

    void setState(bool state);
    void slot_about();
    void slot_aboutQt();
    void slot_quit();

signals:
    void stateChanged(bool state);
    void errorOccured(const QString & errorMessage);
    void incomingMessage();

protected:
    void closeEvent(QCloseEvent * event);

private slots:
    void messagesButtonClicked();

private:
    const PathProvider & pathProvider;
    QHBoxLayout * mainLayout;
    QVBoxLayout * leftPanel;
    QPushButton * connectBtn;
    QPushButton * settingsBtn;
    QPushButton * messagesButton;
    QGroupBox * stateGroup;
    QVBoxLayout * rightPanel;
    QTableView * stateTbl;
    IAIndicator * ind;
    QGridLayout * downPanel;
    QLabel * freeMbLbl;
    QLabel * cashLbl;
    QLabel * freeMbValueLbl;
    QLabel * cashValueLbl;
    IATrayIcon * sysTrayIcon;
    MessageHistoryWindow * messagesWindow;

    QIcon logoIcon;
    QIcon onlineIcon;
    QIcon offlineIcon;

    QAction * connectAction;
    QAction * disconnectAction;
    QAction * settingsAction;
    QAction * hideAction;
    QAction * showAction;
    QAction * aboutAction;
    QAction * aboutQtAction;
    QAction * quitAction;

    QSettings & settings;
    QStringList dirNames;
    QMap<QString, int> dirIndex;

    IAStatsModel * statsModel;
    MessagesModel * messagesModel;

    IAFactory iaFactory;
    IAPtr protoPtr;

    bool disconnectedByUser;
    bool firstTime;

    void createActions();
    void createTray();
    void displayMessage(const QString & message);

    void onStatusChanged(bool status);
    void onStatChanged(const LOADSTAT & loadStat);
    void onInfo(const std::string & message, int infoType, int showTime, int sendTime);
    void onError(const std::string & message, int errorCode);
    void onDirNames(const std::vector<std::string> & dirNames);

    bool eventFilter(QObject *, QEvent *);

};

#endif
