/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef WIN32
#include <winsock2.h>
#endif

#include <cstdlib>
#include <iostream>

#include <QtCore>
#include <QApplication>
#include <QTranslator>
#include <QLocale>

#include "platform_path_provider.h"
#include "ia_main_window.h"

int main(int argc, char ** argv)
{
#ifdef WIN32
    WSADATA m_wsaData;
    WSAStartup(MAKEWORD(2, 0), &m_wsaData);
#endif
    PlatformPathProvider pathProvider("qia");
#ifdef DEBUG
    std::cerr << "Translations dir: " << pathProvider.getResourcePath("translations") << std::endl;
    std::cerr << "Images dir:" << pathProvider.getResourcePath("images") << std::endl;
    std::cerr << "Data dir: " << pathProvider.getDataDir() << std::endl;
#endif
    QApplication app(argc, argv);
    QSettings settings("STGSoft", "qia");

    QString localeName(settings.value("localeName", "default").toString());
    QLocale locale;

    if (localeName == "" || localeName == "default") {
        QString lc_messages(getenv("LC_MESSAGES"));
        QString lang(getenv("LANG"));
        QString lc_all(getenv("LC_ALL"));

#ifdef DEBUG
        std::cerr << "LC_ALL = '" << lc_all.toStdString() << "'" << std::endl;
        std::cerr << "LC_LANG = '" << lang.toStdString() << "'" << std::endl;
        std::cerr << "LC_MESSAGES = '" << lc_messages.toStdString() << "'" << std::endl;
#endif

        if (!lc_all.isEmpty() && lc_all != "\"\"") {
            localeName = QLocale(lc_all).name();
        } else if (!lc_messages.isEmpty() && lc_messages != "\"\"") {
            localeName = QLocale(lc_messages).name();
        } else if (!lang.isEmpty() && lang != "\"\"") {
            localeName = QLocale(lang).name();
        } else {
            localeName = QLocale::system().name();
        }
    }

#ifdef DEBUG
    std::cerr << "Locale name: " << localeName.toStdString() << std::endl; 
#endif

    QTranslator translator;
    QTranslator qtTranslator;

    translator.load("qia_" + localeName, pathProvider.getResourcePath("translations").string().c_str());
    qtTranslator.load("qt_" + localeName,
            QLibraryInfo::location(QLibraryInfo::TranslationsPath));

    app.installTranslator(&translator);
    app.installTranslator(&qtTranslator);

    app.setQuitOnLastWindowClosed(false);

    IAMainWindow win(settings, pathProvider);

    //hide on start
    if(settings.value("minimize_on_start","fasle").toBool()){
        win.hide();
    }else{
        win.show();
    }
    int res =  app.exec();
#ifdef WIN32
    WSACleanup();
#endif
    return res;
}
